######################################
# Statistics for data science: R 
# 
# http://didawiki.di.unipi.it/doku.php/mds/sds/
######################################

# [CTRL-Enter] to execute a line

###### Script Lesson 11: Moments, functions of random variables ##########

#### Moments

# positive skewness and platykurtic
x = rbinom(10000, 50, 0.1)
plot(table(x)/length(x), type='p')
mu = mean(x) # sample mean
sigma = sd(x) # sample sd
mu3 = mean((x-mu)^3)/sigma^3 # sample skewness
mu3
# or using a library
library(moments)
skewness(x)
kurtosis(x)

# set working dir to current source file
# install.packages("rstudioapi")
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
# Data from Human Mortality Database https://www.mortality.org (require registration)
#library(HMDHFDplus)
#mortalitycount = readHMDweb(CNTRY='ITA', item='Deaths_1x1', username='<username>', password='<pwd>')
#save(mortalitycount, file="smd09.Rdata")
load(file="sds11.Rdata")
View(mortalitycount)

# negative skewness and leptokurtic
year = 2015
ndeaths = mortalitycount[mortalitycount$Year==year,]$Total
ages = mortalitycount[mortalitycount$Year==year,]$Age
plot(ages, ndeaths)
x = rep(ages, ndeaths)
mean(x)
plot(table(x)/length(x), xlab="ages", ylab="pmf")
skewness(x)
kurtosis(x)

# cross-entropy and KL divergence (data without covariates/features)
n = 1000 # try with n = 100
data = rgeom(n, 0.7)
px = table(data)/length(data) # empirical distribution
px

test = function(p) {
  distinct = as.numeric(names(px))
  predictions = dgeom(distinct, p) # same predictions for all data
  py = predictions/sum(predictions) 
  cross_entropy = -sum(px*log(py))
  entropy = -sum(px*log(px))
  kl = cross_entropy - entropy
  kl
}

test(0.7)
p = seq(0.6, 0.8, 0.01)
plot(p, sapply(p, test)) # minimize kl iff minimize cross_entropy
